import jwt
import os
import random
import subprocess
import string
from boto3 import Session, client
from botocore.session import get_session
from botocore.credentials import DeferredRefreshableCredentials
from dataEngine import Common, AppConfig, Logger
from .Driver import ConnectorType

AWS_REGION = 'eu-central-1'
# Identity pool
AWS_COGNITO_IP = 'eu-central-1:8b01990e-357b-4842-ac10-4d6db47a59b4'
# Identity pool client
AWS_COGNITO_IP_CLIENT = '78ivskg3alsv9fojsvrvl4svq4'
AWS_COGNITO_IP_ISSUER = 'cognito-idp.eu-central-1.amazonaws.com/eu-central-1_4mprRMCca'
WDA_ROLE = 'arn:aws:iam::572230114905:role/KinesisCrossAccount'
AWS_STREAM = 'wda-events'
AWS_STREAM_STAGE = 'wda-events-stage'
AWS_STREAM_STABLE = 'wda-events-stable'
if os.path.exists("/root/.aws/config"):
    AWS_SECRET_ACCESS_KEY = subprocess.getoutput("cat /root/.aws/config  | grep aws_secret_access_key | cut -d' ' -f3")
    AWS_ACCESS_KEY_ID = subprocess.getoutput("cat /root/.aws/config  | grep aws_access_key_id | cut -d' ' -f3")
else:
    AWS_SECRET_ACCESS_KEY = None
    AWS_ACCESS_KEY_ID = None


logger = Logger.get()


# Random password required to sign up in Cognito User Pool
# Requirements: https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-settings-policies.html
# - Minimum length, which must be at least 6 characters but fewer than 99 characters
# - Require numbers
# - Require a special character from this set: ^ $ * . [ ] { } ( ) ? " ! @ # % & / \ , > < ' : ; | _ ~ `
# - Require uppercase Basic Latin letters
# - Require lowercase Basic Latin letters
def get_secure_random_password():
    alphabet = string.ascii_letters + string.digits + string.punctuation
    while True:
        password = ''.join(random.SystemRandom().choice(alphabet) for _ in range(16))
        if (any(c.islower() for c in password)
                and any(c.isupper() for c in password)
                and any(c.isdigit() for c in password)
                and any(c in string.punctuation for c in password)):
            return password


# Auto-refreshable credential provider
class CredentialProvider():
    CANONICAL_NAME = "credential-provider"

    def __init__(self):
        self.TOKEN_AUTH = None
        self.COMPANY = None


    def get_token(self):
        return self.TOKEN_AUTH


    def get_company(self):
        return self.COMPANY if self.COMPANY else ""


    def load(self):
        # These creds will be automatically refreshed using the _refreh method if the current creds are going to expire
        creds = DeferredRefreshableCredentials(refresh_using=self._refresh, method="sts-assume-role",)
        return creds


    def METHOD():
        return "DeferredRefreshableCredentials"

    def _refresh(self):
        if Common.PBX_SERIAL == "dummypbxserial" and AppConfig.get("driver") == ConnectorType.CwConnector:
            return
        logger.debug("Start refresh token")
        # Refresh AWS credtials
        response = self._aws_cred_refresh()
        credentials = {
            "access_key": response.get("AccessKeyId"),
            "secret_key": response.get("SecretAccessKey"),
            "token": response.get("SessionToken"),
            "expiry_time": response.get("Expiration").isoformat(),
        }
        return credentials


    def _get_creds_cognito(self):
        if AppConfig.get("driver") == ConnectorType.FsConnector:
            return None, None, None
        # Identity Provider Authorization (e.g. JWT for HTTP requests)
        identityProvider = client(
            'cognito-idp',
            region_name=AWS_REGION
        )

        def _initiate_auth(identityProvider):
            return identityProvider.initiate_auth(
                AuthFlow='CUSTOM_AUTH',
                AuthParameters={
                    'USERNAME': Common.PBX_SERIAL
                },
                ClientId=AWS_COGNITO_IP_CLIENT
            )
        try:
            # Initiate Identity Provider authentication flow
            authResponse = _initiate_auth(identityProvider)
        except identityProvider.exceptions.UserNotFoundException:
            # Register the PBX in the Cognito User Pool
            identityProvider.sign_up(
                ClientId=AWS_COGNITO_IP_CLIENT,
                Password=AWS.PBX_RANDOM_PWD,
                Username=Common.PBX_SERIAL,
                ValidationData=[
                    {
                        'Name': 'key',
                        'Value': Common.PBX_KEY,
                    },
                ],
            )
            # Initiate Identity Provider authentication flow
            authResponse = _initiate_auth(identityProvider)
        # Get auth result - IdToken, AccessToken etc.
        authResult = identityProvider.respond_to_auth_challenge(
            ChallengeName='CUSTOM_CHALLENGE',
            ChallengeResponses={
                'USERNAME': Common.PBX_SERIAL,
                'ANSWER': Common.PBX_KEY
            },
            ClientId=AWS_COGNITO_IP_CLIENT,
            Session=authResponse['Session']
        )
        if 'AuthenticationResult' not in authResult:
            logger.error('Authorization failed: {}'.format(authResult))
            AppConfig.set("stop", True)
            return

        self.TOKEN_AUTH = authResult['AuthenticationResult']['IdToken']
        self.COMPANY = jwt.decode(self.TOKEN_AUTH, verify=False).get('custom:company_id')

        # logger.error("Cognito Credentials:\n{}\n\n\n\n".format(authResult['AuthenticationResult']))
        ##########################################
        # Identity Authorization (e.g. Token to access AWS services from PBX role like S3, Kinesis)
        ##########################################
        clientIdentity = client('cognito-identity', region_name=AWS_REGION)
        # Get Cognito Identity ID
        identityResponse = clientIdentity.get_id(
            IdentityPoolId=AWS_COGNITO_IP,
            Logins={AWS_COGNITO_IP_ISSUER:authResult['AuthenticationResult']['IdToken']}
        )
        # Get credentials for the Identity
        credentialsResponse = clientIdentity.get_credentials_for_identity(
            IdentityId=identityResponse['IdentityId'],
            Logins={AWS_COGNITO_IP_ISSUER:authResult['AuthenticationResult']['IdToken']}
        )
        # logger.error("AWS Credentials:\n{}\n\n\n\n".format(credentialsResponse["Credentials"]))
        # return credentialsResponse["Credentials"]
        AccessKeyId = credentialsResponse['Credentials']['AccessKeyId']
        SecretKey = credentialsResponse['Credentials']['SecretKey']
        SessionToken = credentialsResponse['Credentials']['SessionToken']
        return AccessKeyId, SecretKey, SessionToken


    def _aws_cred_refresh(self):
        AccessKeyId, SecretKey, SessionToken = self._get_creds_cognito()

        ##########################################
        # Cross account authorization (e.g. access to resource from WDA account using CPA account credentials)
        ##########################################
        stsClient = client(
            'sts',
            region_name=AWS_REGION,
            aws_access_key_id=AccessKeyId,
            aws_secret_access_key=SecretKey,
            aws_session_token=SessionToken
        ) if AccessKeyId and SecretKey and SessionToken else \
            client('sts',region_name=AWS_REGION)

        stsCredentialsResponse = stsClient.assume_role(
            RoleArn=WDA_ROLE,
            RoleSessionName="KinesisCrossAccount"
        )
        # logger.error("STS Credentials:\n{}\n\n\n\n".format(stsCredentialsResponse["Credentials"]))
        return stsCredentialsResponse['Credentials']



def get_token():
    return AWS.AWS_CREDENTIAL_PROVIDER.get_token()


def get_company():
    return AWS.AWS_CREDENTIAL_PROVIDER.get_company()


def push_data(payload, stream):
    if payload is None:
        logger.warning("Something went wrong, we try to send wrong payload")
        return None
    payload_bytes = payload.encode('utf-8')
    return AWS.KINESIS_CLIENT.put_record(StreamName=stream,
                                         Data=payload_bytes,
                                         PartitionKey=Common.PBX_SERIAL if AppConfig.get("driver") == ConnectorType.CwConnector else Common.PBX_HOSTNAME)


def init_client():
    if AppConfig.get("driver") != ConnectorType.CwConnector:
        global WDA_ROLE
        WDA_ROLE = 'arn:aws:iam::572230114905:role/KinesisforVoxloudRole'
    session = get_session()
    cred_provider = session.get_component('credential_provider')
    cred_provider.insert_before('env', AWS.AWS_CREDENTIAL_PROVIDER)
    session.set_config_variable("region", AWS_REGION)
    autorefresh_session = Session(botocore_session=session)
    AWS.KINESIS_CLIENT = autorefresh_session.client('kinesis', region_name=AWS_REGION)
    AWS.AWS_CREDENTIAL_PROVIDER._refresh()


class Aws():
    def __init__(self):
        if AWS_SECRET_ACCESS_KEY:
            os.environ["AWS_SECRET_ACCESS_KEY"] = AWS_SECRET_ACCESS_KEY
        if AWS_ACCESS_KEY_ID:
            os.environ["AWS_ACCESS_KEY_ID"] = AWS_ACCESS_KEY_ID

        # AWS Credential Provider object
        self.AWS_CREDENTIAL_PROVIDER = CredentialProvider()
        self.PBX_RANDOM_PWD = get_secure_random_password()


AWS = Aws()