import json
import os
import subprocess

lic = {
    "xbees": {
        "is_active": None,
        "check_cmd": "sed '21q;d' /rw2/etc/sf2"},
    "xhoppers": {
        "is_active": None,
        "check_cmd": "sed '19q;d' /rw2/etc/sf2 |awk -F '|' '{print $11}'"}
    }

# PBX
PBX_SERIAL = os.environ.get("PBX_SERIAL", None)
if not PBX_SERIAL:
    res, PBX_SERIAL = subprocess.getstatusoutput("serial")
    if res != 0:
        PBX_SERIAL = "dummypbxserial"

PBX_KEY = os.environ.get("PBX_KEY", None)
if not PBX_KEY:
    try:
        PBX_KEY = subprocess.getoutput("sed '18q;d' /rw2/etc/sf2")
    except FileNotFoundError:
        PBX_KEY = "dummypbxkey"

PBX_HOSTNAME = os.environ.get("PBX_HOSTNAME", None)
if not PBX_HOSTNAME:
    try:
        PBX_HOSTNAME = subprocess.getoutput("hostname")
    except Exception:
        PBX_HOSTNAME = "dummypbxhost"


def str_to_bool(value):
    if not value:
        return False
    if isinstance(value, bool):
        return value
    if value.lower() in ('false', 'f', '0', 'no', 'n', 'off'):
        return False
    elif value.lower() in ('true', 't', '1', 'yes', 'y', 'on'):
        return True
    raise ValueError(f'{value} is not a valid boolean value')


def check_license(service, force_load=None):
    if service not in lic:
        return
    if lic[service]["is_active"] is None or force_load:
        try:
            if subprocess.getoutput(lic[service]["check_cmd"]) == "1":
                lic[service]["is_active"] = True
            else:
                lic[service]["is_active"] = False
        except FileNotFoundError:
            return None
    return lic[service]["is_active"]


def reload(param):
    if param == "license":
        for service in lic:
            check_license(service, True)


class PbxDataObject:
    __slots__ = ()

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def get(self, param):
        return getattr(self, param, None)

    def set(self, param, value):
        if value:
            setattr(self, param, value)

    def __bool__(self):
        return True


class PbxDataEncoder(json.JSONEncoder):
    __slots__ = ()

    def default(self, obj):
        if isinstance(obj, PbxDataObject):
            return {(key[1:] if key[0] == "_" else key):(val if val else None) for key,val in obj.__dict__.items()}
        return json.JSONEncoder.default(self, obj)
