import socket
import time
from datetime import datetime
from threading import Thread
from dataEngine import AppConfig, Logger
from dataEngine.Driver import BaseMock

logger = Logger.get()


class Mock(BaseMock):
    def __init__(self):
        self.thread = Thread(target=self._listen_socket)
        self.exit = False
        self.connection = None
        self.loggedin = False


    def init_connection(self):
        if AppConfig.isTest():
            self.thread.start()


    def close_connection(self):
        self.exit = True
        if self.thread.is_alive():
            self.thread.join()


    def send(self, data):
        if self.connection:
            self.connection.sendall(str.encode(data))


    def get_logged_state(self):
        return self.loggedin


    def _listen_socket(self):
        port = AppConfig.get("port")
        logger.debug("Start to use port: {} for pseudo AMI".format(port))
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
            try:
                sock.bind(("127.0.0.1", port))
            except OSError as e:
                logger.error("Can't bind to port {}".format(port))
                AppConfig.set("stop", True)
                return
            sock.listen()
            sock.settimeout(2.0)
            try:
                conn, addr = sock.accept()
            except socket.timeout as e:
                logger.error("Nobody connect to port {} by timeout".format(port))
                return
            with conn:
                conn.sendall(str.encode("Callweaver Call Manager/1.3\r\n"))
                self.connection = conn
                while True:
                    if self.exit:
                        break
                    data_byte = conn.recv(1024)
                    if not data_byte:
                        continue
                    data = {item.split(":")[0]:item.split(":")[1].replace(" ", "") for item in data_byte.decode("utf-8").split("\r\n") if item}
                    if data["Action"] == "Login":
                        conn.sendall(str.encode("Response: Success\r\n"
                                                "ActionID: {}\r\n"
                                                "Message: Authentication accepted\r\n\r\n".format(data["ActionID"])))
                        time.sleep(0.2)
                        conn.sendall(str.encode("Event: FullyBooted\r\n"
                                                "Privilege: system,all\r\n"
                                                "Status: Fully Booted\r\n\r\n"))
                        time.sleep(0.2)
                        self.loggedin = True
                    elif data["Action"] == "Ping":
                        conn.sendall(str.encode("Response: Success\r\n"
                                                "ActionID: {}\r\n".format(data["ActionID"]) +
                                                "Ping: Pong\r\n"
                                                "Timestamp: {}.000000\r\n\r\n".format(int(datetime.timestamp(datetime.now())))))
                    elif data["Action"] == "Logoff":
                        conn.sendall(str.encode("Response: Goodbye\r\n"
                                                "ActionID: {}\r\n".format(data["ActionID"]) +
                                                "Message: Thanks for all the fish.."))
                        break
            self.connection = None
