from dataEngine.fs import EventHandlers
from dataEngine.Driver import BaseConnector
from dataEngine import Logger, AppConfig
import greenswitch
import gevent
import os
from threading import Thread

logger = Logger.get()

HOST = os.environ.get("FS_HOST", "127.0.0.1")
PASS = os.environ.get("FS_PASS", "ClueCon")
PORT = int(os.environ.get("FS_PORT", "8021"))


class Connector(BaseConnector):
    FS = None
    thread = None

    def __init__(self):
        super(BaseConnector, self).__init__()
        self.thread = Thread(target=self._run)

    def _run(self):
        self.FS = greenswitch.InboundESL(host=HOST, port=PORT, password=PASS)
        try:
            self.FS.connect()
        except Exception:
            logger.exception("Can't connect to Freeswitch!")
            AppConfig.set("stop", True)
            return

        if self.FS.connected:
            logger.info('Connected to FreeSWITCH!')
        else:
            logger.error("The connection to the Freeswitch has failed :(")
            AppConfig.set("stop", True)
            return

        self.FS.register_handle("*", EventHandlers.event_listener)
        self.FS.send('EVENTS PLAIN ALL')
        while True:
            if AppConfig.isStopped():
                break
            gevent.sleep(1)

    def init_connection(self, cleanup_all_data):
        self.thread.start()

    def close_connection(self):
        if self.thread.is_alive():
            self.thread.join()
